@@ -1,33 +1,69 @@
 """
 PSD Image module.
-"""
 
-from __future__ import annotations
+This module provides the main :py:class:`PSDImage` class, which is the primary
+entry point for users of psd-tools. It represents a complete Photoshop document
+and provides high-level methods for reading, manipulating, and saving PSD/PSB files.
+
+The :py:class:`PSDImage` class wraps the low-level :py:class:`~psd_tools.psd.PSD`
+structure and reconstructs the layer tree from the flat layer list, making it much
+easier to work with layers and groups.
+
+Key functionality:
+
+- **Opening files**: :py:meth:`PSDImage.open` and :py:meth:`PSDImage.new`
+- **Layer access**: Iterate, index, and search layers
+- **Compositing**: Render layers to PIL Images via :py:meth:`~PSDImage.composite`
+- **Saving**: Write modified documents back to PSD format
+- **Metadata access**: Document properties, ICC profiles, resolution, etc.
+
+Example usage::
+
+    from psd_tools import PSDImage
+
+    # Open a PSD file
+    psd = PSDImage.open('document.psd')
+
+    # Access document properties
+    print(f"Size: {psd.width}x{psd.height}")
+    print(f"Color mode: {psd.color_mode}")
+
+    # Iterate through layers
+    for layer in psd:
+        print(f"{layer.name}: {layer.kind}")
+
+    # Access layers by index
+    layer = psd[0]  # First layer
+
+    # Modify layers
+    layer.visible = False
+    layer.opacity = 128
+
+    # Composite to image
+    image = psd.composite()
+    image.save('output.png')
+
+    # Save changes
+    psd.save('modified.psd')
+
+The class inherits from :py:class:`~psd_tools.api.layers.GroupMixin`, providing
+group-like behavior for accessing child layers.
+"""
 
 import logging
 import os
-from typing import Any, BinaryIO, Callable, Literal
+from typing import Any, BinaryIO, Callable, Iterable, Literal, Optional, Union
 
 try:
-    from typing import Self
+    from typing import Self  # type: ignore[attr-defined]
 except ImportError:
     from typing_extensions import Self
 
 import numpy as np
-from PIL.Image import Image as PILImage
+from PIL import Image
 
-from psd_tools.api import adjustments
-from psd_tools.api.layers import (
-    Artboard,
-    FillLayer,
-    Group,
-    GroupMixin,
-    PixelLayer,
-    ShapeLayer,
-    SmartObjectLayer,
-    TypeLayer,
-)
-from psd_tools.api.pil_io import get_pil_channels, get_pil_mode
+from psd_tools.api import adjustments, layers, numpy_io, pil_io
+from psd_tools.api.protocols import PSDProtocol
 from psd_tools.constants import (
     BlendMode,
     ChannelID,
@@ -38,24 +74,25 @@ from psd_tools.constants import (
     SectionDivider,
     Tag,
 )
-from psd_tools.psd import (
-    PSD,
+from psd_tools.psd.document import PSD
+from psd_tools.psd.header import FileHeader
+from psd_tools.psd.image_data import ImageData
+from psd_tools.psd.image_resources import ImageResources
+from psd_tools.psd.layer_and_mask import (
     ChannelImageData,
-    FileHeader,
     GlobalLayerMaskInfo,
-    ImageData,
-    ImageResources,
     LayerInfo,
     LayerRecords,
-    TaggedBlocks,
 )
+from psd_tools.psd.patterns import Patterns
+from psd_tools.psd.tagged_blocks import TaggedBlocks
 
 logger = logging.getLogger(__name__)
 
 
-class PSDImage(GroupMixin):
+class PSDImage(layers.GroupMixin, PSDProtocol):
     """
-    Photoshop PSD/PSB file object.
+    Photoshop PSD/PSB document.
 
     The low-level data structure is accessible at :py:attr:`PSDImage._record`.
 
@@ -63,21 +100,22 @@ class PSDImage(GroupMixin):
 
         from psd_tools import PSDImage
 
-        psd = PSDImage.open('example.psd')
-        image = psd.compose()
+        psdimage = PSDImage.open('example.psd')
+        image = psdimage.composite()
 
-        for layer in psd:
-            layer_image = layer.compose()
+        for layer in psdimage:
+            layer_image = layer.composite()
     """
 
     def __init__(self, data: PSD):
-        from psd_tools.api.layers import Layer
-
-        assert isinstance(data, PSD)
+        if not isinstance(data, PSD):
+            raise TypeError(f"Expected PSD instance, got {type(data).__name__}")
         self._record = data
-        self._layers: list[Layer] = []
+        self._layers: list[layers.Layer] = []
         self._compatibility_mode = CompatibilityMode.DEFAULT
-        self._updated_layers = False  # Flag to check if the layer tree is edited.
+        self._updated: bool = False  # Flag to check if the layer tree is edited.
+
+        self._psd = self  # For GroupMixin protocol compatibility.
         self._init()
 
     @classmethod
@@ -88,7 +126,7 @@ class PSDImage(GroupMixin):
         color: int = 0,
         depth: Literal[8, 16, 32] = 8,
         **kwargs: Any,
-    ):
+    ) -> Self:
         """
         Create a new PSD document.
 
@@ -109,9 +147,11 @@ class PSDImage(GroupMixin):
         )
 
     @classmethod
-    def frompil(cls, image: PILImage, compression=Compression.RLE) -> Self:
+    def frompil(
+        cls, image: Image.Image, compression: Compression = Compression.RLE
+    ) -> Self:
         """
-        Create a new PSD document from PIL Image.
+        Create a new layer-less PSD document from PIL Image.
 
         :param image: PIL Image object.
         :param compression: ImageData compression option. See
@@ -132,7 +172,7 @@ class PSDImage(GroupMixin):
         )
 
     @classmethod
-    def open(cls, fp: BinaryIO | str | bytes | os.PathLike, **kwargs: Any) -> Self:
+    def open(cls, fp: Union[BinaryIO, str, bytes, os.PathLike], **kwargs: Any) -> Self:
         """
         Open a PSD document.
 
@@ -148,7 +188,12 @@ class PSDImage(GroupMixin):
             self = cls(PSD.read(fp, **kwargs))
         return self
 
-    def save(self, fp: BinaryIO | str | bytes | os.PathLike, mode: str = "wb", **kwargs: Any) -> None:
+    def save(
+        self,
+        fp: Union[BinaryIO, str, bytes, os.PathLike],
+        mode: str = "wb",
+        **kwargs: Any,
+    ) -> None:
         """
         Save the PSD file. Updates the ImageData section if the layer structure has been updated.
 
@@ -157,22 +202,32 @@ class PSDImage(GroupMixin):
             default 'macroman'.
         :param mode: file open mode, default 'wb'.
         """
-
-        self._update_record()
-
-        if self._updated_layers:
-            composited_psd = self.composite(force=True)
-            self._record.image_data.set_data([channel.tobytes() for channel in composited_psd.split()], self._record.header)
+        if self.is_updated():
+            # Update the preview image if the layer structure has been changed.
+            # TODO: Fill in a white background for the given color mode on failure.
+            # TODO: Set a `has_composite` flag in VersionInfo resource.
+            try:
+                composited_psd = self.composite()
+                self._record.image_data.set_data(
+                    [channel.tobytes() for channel in composited_psd.split()],
+                    self._record.header,
+                )
+            except ImportError as e:
+                logger.warning(
+                    "Failed to update preview image: %s. "
+                    "Install composite dependencies with: pip install 'psd-tools[composite]'",
+                    e,
+                )
 
         if isinstance(fp, (str, bytes, os.PathLike)):
             with open(fp, mode) as f:
-                self._record.write(f, **kwargs)
+                self._record.write(f, **kwargs)  # type: ignore[arg-type]
         else:
-            self._record.write(fp, **kwargs)
+            self._record.write(fp, **kwargs)  # type: ignore[arg-type]
 
     def topil(
-        self, channel: int | ChannelID | None = None, apply_icc: bool = True
-    ) -> PILImage | None:
+        self, channel: Union[int, ChannelID, None] = None, apply_icc: bool = True
+    ) -> Union[Image.Image, None]:
         """
         Get PIL Image.
 
@@ -183,16 +238,12 @@ class PSDImage(GroupMixin):
         :return: :py:class:`PIL.Image`, or `None` if the composed image is not
             available.
         """
-        from .pil_io import convert_image_data_to_pil
-
-        self._update_record()
-
         if self.has_preview():
-            return convert_image_data_to_pil(self, channel, apply_icc)
+            return pil_io.convert_image_data_to_pil(self, channel, apply_icc)
         return None
 
     def numpy(
-        self, channel: Literal["color", "shape", "alpha", "mask"] | None = None
+        self, channel: Optional[Literal["color", "shape", "alpha", "mask"]] = None
     ) -> np.ndarray:
         """
         Get NumPy array of the layer.
@@ -201,20 +252,20 @@ class PSDImage(GroupMixin):
             'shape', 'alpha', or 'mask'. Default is 'color+alpha'.
         :return: :py:class:`numpy.ndarray`
         """
-        from .numpy_io import get_array
-
-        return get_array(self, channel)
+        array = numpy_io.get_array(self, channel)
+        assert array is not None
+        return array
 
     def composite(
         self,
-        viewport: tuple[int, int, int, int] | None = None,
+        viewport: Optional[tuple[int, int, int, int]] = None,
         force: bool = False,
-        color: float | tuple[float, ...] = 1.0,
-        alpha: float = 0.0,
-        layer_filter: Callable | None = None,
+        color: Union[float, tuple[float, ...], np.ndarray, None] = 1.0,
+        alpha: Union[float, np.ndarray] = 0.0,
+        layer_filter: Optional[Callable] = None,
         ignore_preview: bool = False,
         apply_icc: bool = True,
-    ):
+    ) -> Image.Image:
         """
         Composite the PSD image.
 
@@ -234,39 +285,45 @@ class PSDImage(GroupMixin):
         """
         from psd_tools.composite import composite_pil
 
-        self._update_record()
-
         if (
             not (ignore_preview or force or layer_filter)
             and self.has_preview()
-            and not self._updated_layers
+            and not self.is_updated()
         ):
-            return self.topil(apply_icc=apply_icc)
-        return composite_pil(
-            self, color, alpha, viewport, layer_filter, force, apply_icc=apply_icc
+            result = self.topil(apply_icc=apply_icc)
+            if result is None:
+                raise ValueError("Failed to composite PSD image from preview")
+            return result
+        result = composite_pil(
+            self,
+            color if color is not None else 1.0,
+            alpha,
+            viewport,
+            layer_filter,
+            force,
+            apply_icc=apply_icc,
         )
+        if result is None:
+            raise ValueError("Failed to composite PSD image")
+        return result
 
-    def is_visible(self) -> bool:
+    def _mark_updated(self) -> None:
+        """Mark the layer tree as updated."""
+        self._updated = True
+
+    def is_updated(self) -> bool:
         """
-        Returns visibility of the element.
+        Returns whether the layer tree has been updated.
 
         :return: `bool`
         """
-        return self.visible
+        return self._updated
 
     @property
     def parent(self) -> None:
         """Parent of this layer."""
         return None
 
-    def is_group(self) -> bool:
-        """
-        Return True if the layer is a group.
-
-        :return: `bool`
-        """
-        return isinstance(self, GroupMixin)
-
     def has_preview(self) -> bool:
         """
         Returns if the document has real merged data. When True, `topil()`
@@ -466,7 +523,7 @@ class PSDImage(GroupMixin):
         return self._record.image_resources
 
     @property
-    def tagged_blocks(self) -> TaggedBlocks | None:
+    def tagged_blocks(self) -> Optional[TaggedBlocks]:
         """
         Document tagged blocks that is a dict-like container of settings.
 
@@ -488,19 +545,29 @@ class PSDImage(GroupMixin):
         """
         Set the compositing and layer organization compatibility mode. Writable.
 
+        This property checks whether the PSD file is compatible with
+        specific authoring tools, such as Photoshop or CLIP Studio Paint.
+        Different authoring tools may have different ways of handling layers,
+        such as the use of clipping masks for groups.
+        Default is Photoshop compatibility mode.
+
         :return: :py:class:`~psd_tools.constants.CompatibilityMode`
         """
         return self._compatibility_mode
 
     @compatibility_mode.setter
     def compatibility_mode(self, value: CompatibilityMode) -> None:
+        if self._compatibility_mode != value:
+            self._mark_updated()
         self._compatibility_mode = value
-        self._compute_clipping_layers()
 
     @property
     def pil_mode(self) -> str:
-        alpha = self.channels - get_pil_channels(get_pil_mode(self.color_mode))
-        return get_pil_mode(self.color_mode, alpha > 0)
+        alpha = self.channels - pil_io.get_pil_channels(
+            pil_io.get_pil_mode(self.color_mode)
+        )
+        # TODO: Check when alpha > 1; Photoshop allows multiple alpha channels.
+        return pil_io.get_pil_mode(self.color_mode, alpha > 0)
 
     def has_thumbnail(self) -> bool:
         """True if the PSDImage has a thumbnail resource."""
@@ -509,23 +576,91 @@ class PSDImage(GroupMixin):
             or Resource.THUMBNAIL_RESOURCE_PS4 in self.image_resources
         )
 
-    def thumbnail(self) -> PILImage | None:
+    def thumbnail(self) -> Optional[Image.Image]:
         """
         Returns a thumbnail image in PIL.Image. When the file does not
         contain an embedded thumbnail image, returns None.
         """
-        from .pil_io import convert_thumbnail_to_pil
-
         if Resource.THUMBNAIL_RESOURCE in self.image_resources:
-            return convert_thumbnail_to_pil(
+            return pil_io.convert_thumbnail_to_pil(
                 self.image_resources.get_data(Resource.THUMBNAIL_RESOURCE)
             )
         elif Resource.THUMBNAIL_RESOURCE_PS4 in self.image_resources:
-            return convert_thumbnail_to_pil(
+            return pil_io.convert_thumbnail_to_pil(
                 self.image_resources.get_data(Resource.THUMBNAIL_RESOURCE_PS4)
             )
         return None
 
+    # Editing API
+    def create_pixel_layer(
+        self,
+        image: Image.Image,
+        name: str = "Layer",
+        top: int = 0,
+        left: int = 0,
+        compression: Compression = Compression.RLE,
+        opacity: int = 255,
+        blend_mode: BlendMode = BlendMode.NORMAL,
+    ) -> layers.PixelLayer:
+        """
+        Create a new pixel layer and add it to the PSDImage.
+
+        Example::
+
+            psdimage = PSDImage.new("RGB", (640, 480))
+            layer = psdimage.create_pixel_layer(image, name='Layer 1')
+
+        :param name: Name of the new layer.
+        :param image: PIL Image object.
+        :param top: Top coordinate of the new layer.
+        :param left: Left coordinate of the new layer.
+        :param compression: Compression method for the layer image data.
+        :param opacity: Opacity of the new layer (0-255).
+        :param blend_mode: Blend mode of the new layer, default is ``BlendMode.NORMAL``.
+        :return: The created :py:class:`~psd_tools.api.layers.PixelLayer` object.
+        """
+        layer = layers.PixelLayer.frompil(
+            image, parent=self, name=name, top=top, left=left, compression=compression
+        )
+        layer.opacity = opacity
+        layer.blend_mode = blend_mode
+        self._mark_updated()
+        return layer
+
+    def create_group(
+        self,
+        layer_list: Optional[Iterable[layers.Layer]] = None,
+        name: str = "Group",
+        opacity: int = 255,
+        blend_mode: BlendMode = BlendMode.PASS_THROUGH,
+        open_folder: bool = True,
+    ) -> layers.Group:
+        """
+        Create a new group layer and add it to the PSDImage.
+
+        Example::
+
+            group = psdimage.create_group(name='New Group')
+            group.append(psdimage.create_pixel_layer(image, name='Layer in Group'))
+
+        :param layer_list: Optional list of layers to add to the group.
+        :param name: Name of the new group.
+        :param opacity: Opacity of the new layer (0-255).
+        :param blend_mode: Blend mode of the new layer, default is ``BlendMode.PASS_THROUGH``.
+        :param open_folder: Whether the group is an open folder in the Photoshop UI.
+        :return: The created :py:class:`~psd_tools.api.layers.Group` object.
+        """
+        group = layers.Group.new(parent=self, name=name, open_folder=open_folder)
+        if layer_list:
+            group.extend(layer_list)
+        group.opacity = opacity
+        group.blend_mode = blend_mode
+        self._mark_updated()
+        return group
+
+    # TODO: Add more editing APIs, such as duplicate_layers, resize_canvas, etc.
+
+    # Private methods
     def __repr__(self) -> str:
         return ("%s(mode=%s size=%dx%d depth=%d channels=%d)") % (
             self.__class__.__name__,
@@ -536,22 +671,20 @@ class PSDImage(GroupMixin):
             self._record.header.channels,
         )
 
-    def _repr_pretty_(self, p, cycle):
+    def _repr_pretty_(self, p: Any, cycle: bool) -> None:
         if cycle:
-            return self.__repr__()
+            p.text(self.__repr__())
+            return
 
-        def _pretty(layer, p):
+        def _pretty(layer: Union[layers.Layer, "PSDImage"], p: Any) -> None:
             p.text(layer.__repr__())
-            if hasattr(layer, "clip_layers"):
-                for clip_layer in layer.clip_layers or []:
-                    p.break_()
-                    p.text(" +  ")
-                    p.pretty(clip_layer)
-            if hasattr(layer, "__iter__"):
+            if isinstance(layer, layers.GroupMixin):
                 with p.indent(2):
                     for idx, child in enumerate(layer):
                         p.break_()
                         p.text("[%d] " % idx)
+                        if child.clipping:
+                            p.text("+")
                         _pretty(child, p)
 
         _pretty(self, p)
@@ -560,17 +693,18 @@ class PSDImage(GroupMixin):
     def _make_header(
         cls, mode: str, size: tuple[int, int], depth: Literal[8, 16, 32] = 8
     ) -> FileHeader:
-        from .pil_io import get_color_mode
-
-        assert depth in (8, 16, 32), "Invalid depth: %d" % (depth)
-        assert size[0] <= 300000, "Width too large > 300,000"
-        assert size[1] <= 300000, "Height too large > 300,000"
+        if depth not in (8, 16, 32):
+            raise ValueError(f"Invalid depth: {depth}. Must be 8, 16, or 32")
+        if size[0] > 300000:
+            raise ValueError(f"Width too large: {size[0]} > 300,000")
+        if size[1] > 300000:
+            raise ValueError(f"Height too large: {size[1]} > 300,000")
         version = 1
         if size[0] > 30000 or size[1] > 30000:
             logger.debug("Width or height larger than 30,000 pixels")
             version = 2
-        color_mode = get_color_mode(mode)
-        alpha = int(mode.upper().endswith("A"))
+        color_mode = pil_io.get_color_mode(mode)
+        alpha = mode.upper().endswith("A")
         channels = ColorMode.channels(color_mode, alpha)
         return FileHeader(
             version=version,
@@ -581,8 +715,10 @@ class PSDImage(GroupMixin):
             color_mode=color_mode,
         )
 
-    def _get_pattern(self, pattern_id):
+    def _get_pattern(self, pattern_id: str) -> Optional[Any]:
         """Get pattern item by id."""
+        if self.tagged_blocks is None:
+            return None
         for key in (Tag.PATTERNS1, Tag.PATTERNS2, Tag.PATTERNS3):
             if key in self.tagged_blocks:
                 data = self.tagged_blocks.get_data(key)
@@ -591,53 +727,18 @@ class PSDImage(GroupMixin):
                         return pattern
         return None
 
-    def _clear_clipping_layers(self) -> None:
-        for layer in self.descendants():
-            layer._clip_layers = []
-            layer._has_clip_target = True
-
-    def _compute_clipping_layers(self) -> None:
-        self._update_record()
-
-        self._clear_clipping_layers()
-
-        def rec_helper(layer):
-            if not layer.is_group():
-                return
-            stack = []
-            for sublayer in reversed(layer._layers):
-                if sublayer.clipping_layer:
-                    stack.append(sublayer)
-                else:
-                    if sublayer.blend_mode == BlendMode.PASS_THROUGH and (
-                        self.compatibility_mode == CompatibilityMode.PAINT_TOOL_SAI
-                        or self.compatibility_mode
-                        == CompatibilityMode.CLIP_STUDIO_PAINT
-                    ):
-                        for clip_layer in stack:
-                            clip_layer._has_clip_target = False
-                    else:
-                        stack.reverse()
-                        sublayer._clip_layers = stack
-                    stack = []
-                rec_helper(sublayer)
-            for clip_layer in stack:
-                clip_layer._has_clip_target = False
-
-        rec_helper(self)
-
     def _init(self) -> None:
         """Initialize layer structure."""
-        from psd_tools.api.layers import Layer
+        from psd_tools.api import layers
 
-        group_stack: list[Group | Artboard | PSDImage] = [self]
+        group_stack: list[Union[layers.Group, PSDImage]] = [self]
 
         for record, channels in self._record._iter_layers():
             current_group = group_stack[-1]
 
             blocks = record.tagged_blocks
             end_of_group = False
-            layer: Layer | Group | Artboard | PSDImage | None = None
+            layer: Union[layers.Layer, PSDImage, None] = None
             divider = blocks.get_data(Tag.SECTION_DIVIDER_SETTING, None)
             divider = blocks.get_data(Tag.NESTED_SECTION_DIVIDER_SETTING, divider)
             if (
@@ -649,11 +750,11 @@ class PSDImage(GroupMixin):
                 divider.kind is not SectionDivider.OTHER
             ):
                 if divider.kind == SectionDivider.BOUNDING_SECTION_DIVIDER:
-                    layer = Group(  # type: ignore
-                        psd=self,
+                    layer = layers.Group(  # type: ignore
+                        parent=current_group,
+                        # We need to fill in the record and channels later.
                         record=None,  # type: ignore
                         channels=None,  # type: ignore
-                        parent=current_group,
                     )
                     layer._set_bounding_records(record, channels)
                     group_stack.append(layer)
@@ -662,35 +763,38 @@ class PSDImage(GroupMixin):
                     SectionDivider.CLOSED_FOLDER,
                 ):
                     layer = group_stack.pop()
-                    assert not isinstance(layer, PSDImage)
-
+                    if not isinstance(layer, layers.Group):
+                        raise TypeError(
+                            f"Expected Group layer, got {type(layer).__name__}"
+                        )
+                    # Set the record and channels now.
                     layer._record = record
                     layer._channels = channels
+
+                    # If the group is an artboard, convert it.
                     for key in (
                         Tag.ARTBOARD_DATA1,
                         Tag.ARTBOARD_DATA2,
                         Tag.ARTBOARD_DATA3,
                     ):
                         if key in blocks:
-                            layer = Artboard._move(layer)
+                            layer = layers.Artboard._move(layer)
                     end_of_group = True
                 else:
                     logger.warning("Divider %s found." % divider.kind)
             elif Tag.TYPE_TOOL_OBJECT_SETTING in blocks or Tag.TYPE_TOOL_INFO in blocks:
-                layer = TypeLayer(self, record, channels, current_group)
+                layer = layers.TypeLayer(current_group, record, channels)
             elif (
                 Tag.SMART_OBJECT_LAYER_DATA1 in blocks
                 or Tag.SMART_OBJECT_LAYER_DATA2 in blocks
                 or Tag.PLACED_LAYER1 in blocks
                 or Tag.PLACED_LAYER2 in blocks
             ):
-                layer = SmartObjectLayer(self, record, channels, current_group)
+                layer = layers.SmartObjectLayer(current_group, record, channels)
             else:
                 for key in adjustments.TYPES.keys():
                     if key in blocks:
-                        layer = adjustments.TYPES[key](
-                            self, record, channels, current_group
-                        )
+                        layer = adjustments.TYPES[key](current_group, record, channels)
                         break
 
             # If nothing applies, this is either a shape or pixel layer.
@@ -701,61 +805,85 @@ class PSDImage(GroupMixin):
                 or Tag.VECTOR_STROKE_DATA in blocks
                 or Tag.VECTOR_STROKE_CONTENT_DATA in blocks
             )
-            if isinstance(layer, (type(None), FillLayer)) and shape_condition:
-                layer = ShapeLayer(self, record, channels, current_group)
+            if isinstance(layer, (type(None), layers.FillLayer)) and shape_condition:
+                layer = layers.ShapeLayer(current_group, record, channels)
 
             if layer is None:
-                layer = PixelLayer(self, record, channels, current_group)
+                layer = layers.PixelLayer(current_group, record, channels)
 
-            assert layer is not None
+            if layer is None:
+                raise ValueError("Failed to create layer from record")
 
             if not end_of_group:
-                assert not isinstance(layer, PSDImage)
+                if isinstance(layer, PSDImage):
+                    raise TypeError("Cannot add PSDImage as a layer")
                 current_group._layers.append(layer)
 
-        self._compute_clipping_layers()
-
     def _update_record(self) -> None:
         """
-        Compiles the tree layer structure back into records and channels list recursively
+        Compiles the tree layer structure back into records and channels list
+        recursively from the API layer structure.
         """
-
-        if not self._updated_layers:
-            # Skip if nothing is changed.
-            return
-
-        layer_records, channel_image_data = _build_record_tree(self)
-
-        # PSDImage.frompil doesn't create a LayerInfo attribute to LayerAndMaskInformation
-        if not self._record.layer_and_mask_information.layer_info:
+        # Initialize the layer structure information if not present.
+        if self._record.layer_and_mask_information.layer_info is None:
             self._record.layer_and_mask_information.layer_info = LayerInfo()
-
-        if not self._record.layer_and_mask_information.global_layer_mask_info:
+        if self._record.layer_and_mask_information.global_layer_mask_info is None:
             self._record.layer_and_mask_information.global_layer_mask_info = (
                 GlobalLayerMaskInfo()
             )
-
-        if not self._record.layer_and_mask_information.tagged_blocks:
+        if self._record.layer_and_mask_information.tagged_blocks is None:
             self._record.layer_and_mask_information.tagged_blocks = TaggedBlocks()
 
+        # Set layer records and channel image data.
+        layer_records, channel_image_data = _build_record_tree(self)
         layer_info = self._record.layer_and_mask_information.layer_info
         layer_info.layer_records = layer_records
         layer_info.channel_image_data = channel_image_data
         layer_info.layer_count = len(layer_records)
 
+        # Flag as updated.
+        self._mark_updated()
+
+    def _copy_patterns(self, psdimage: PSDProtocol) -> None:
+        """Copy patterns from this psdimage to the target psdimage."""
+        if self.tagged_blocks is None:
+            # Nothing to copy.
+            return
+
+        if psdimage.tagged_blocks is None:
+            logger.debug("Creating tagged blocks for psdimage.")
+            psdimage._record.layer_and_mask_information.tagged_blocks = TaggedBlocks()
+        if psdimage.tagged_blocks is None:
+            raise ValueError("Failed to create tagged blocks for psdimage")
+
+        for tag in self.tagged_blocks.keys():
+            if not isinstance(tag, Tag):
+                raise TypeError(f"Expected Tag instance, got {type(tag).__name__}")
+            if tag in (Tag.PATTERNS1, Tag.PATTERNS2, Tag.PATTERNS3):
+                logger.debug("Copying patterns for tag %s", tag)
+                source_patterns: Patterns = self.tagged_blocks.get_data(tag)
+                target_patterns: Patterns = psdimage.tagged_blocks.get_data(tag)
+                if target_patterns is None:
+                    target_patterns = Patterns()
+                    psdimage.tagged_blocks.set_data(tag, target_patterns)
+
+                target_pattern_ids = {p.pattern_id for p in target_patterns}
+                for pattern in source_patterns:
+                    if pattern.pattern_id not in target_pattern_ids:
+                        target_patterns.append(pattern)
+
 
 def _build_record_tree(
-    layer_group: GroupMixin,
+    layer_group: layers.GroupMixin,
 ) -> tuple[LayerRecords, ChannelImageData]:
     """
     Builds the layer tree structure from records and channels list recursively
     """
-
     layer_records = LayerRecords()
     channel_image_data = ChannelImageData()
 
     for layer in layer_group:
-        if isinstance(layer, (Group, Artboard)):
+        if isinstance(layer, (layers.Group, layers.Artboard)):
             layer_records.append(layer._bounding_record)
             channel_image_data.append(layer._bounding_channels)
 
