@@ -5,23 +5,26 @@ Image data section structure.
 where a composited image is stored. When the file does not contain layers,
 this is the only place pixels are saved.
 """
-from __future__ import absolute_import, unicode_literals
 
 import io
 import logging
+from typing import Any, BinaryIO, Sequence, TypeVar, Union
 
-import attr
+from attrs import define, field
 
 from psd_tools.compression import compress, decompress
 from psd_tools.constants import Compression
+from psd_tools.psd.header import FileHeader
 from psd_tools.psd.base import BaseElement
-from psd_tools.utils import pack, read_fmt, write_bytes, write_fmt
+from psd_tools.psd.bin_utils import pack, read_fmt, write_bytes, write_fmt
 from psd_tools.validators import in_
 
 logger = logging.getLogger(__name__)
 
+T = TypeVar("T", bound="ImageData")
 
-@attr.s(repr=False, slots=True)
+
+@define(repr=False)
 class ImageData(BaseElement):
     """
     Merged channel image data.
@@ -35,27 +38,29 @@ class ImageData(BaseElement):
         `bytes` as compressed in the `compression` flag.
     """
 
-    compression = attr.ib(
+    compression: Compression = field(
         default=Compression.RAW, converter=Compression, validator=in_(Compression)
     )
-    data = attr.ib(default=b"", type=bytes)
+    data: bytes = b""
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         start_pos = fp.tell()
         compression = Compression(read_fmt("H", fp)[0])
         data = fp.read()  # TODO: Parse data here. Need header.
         logger.debug("  read image data, len=%d" % (fp.tell() - start_pos))
         return cls(compression, data)
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         start_pos = fp.tell()
         written = write_fmt(fp, "H", self.compression.value)
         written += write_bytes(fp, self.data)
         logger.debug("  wrote image data, len=%d" % (fp.tell() - start_pos))
         return written
 
-    def get_data(self, header, split=True):
+    def get_data(
+        self, header: FileHeader, split: bool = True
+    ) -> Union[list[bytes], bytes]:
         """
         Get decompressed data.
 
@@ -76,7 +81,7 @@ class ImageData(BaseElement):
                 return [f.read(plane_size) for _ in range(header.channels)]
         return data
 
-    def set_data(self, data, header):
+    def set_data(self, data: Sequence[bytes], header: FileHeader) -> int:
         """
         Set raw data and compress.
 
@@ -97,7 +102,12 @@ class ImageData(BaseElement):
         return len(self.data)
 
     @classmethod
-    def new(cls, header, color=0, compression=Compression.RAW):
+    def new(
+        cls: type[T],
+        header: FileHeader,
+        color: Union[int, Sequence[int]] = 0,
+        compression: Compression = Compression.RAW,
+    ) -> T:
         """
         Create a new image data object.
 
@@ -113,7 +123,7 @@ class ImageData(BaseElement):
                 "Invalid color %s for channel size %d" % (color, header.channels)
             )
         # Bitmap is not supported here.
-        fmt = {8: "B", 16: "H", 32: "I"}.get(header.depth)
+        fmt = {8: "B", 16: "H", 32: "I"}[header.depth]
         data = []
         for i in range(header.channels):
             data.append(pack(fmt, color[i]) * plane_size)
