@@ -1,5 +1,4 @@
-from __future__ import absolute_import, unicode_literals
-
+from typing import Any, Type
 import logging
 from enum import Enum
 
@@ -22,7 +21,7 @@ from ..utils import check_write_read
 logger = logging.getLogger(__name__)
 
 
-def test_empty():
+def test_empty() -> None:
     check_write_read(EmptyElement())
 
 
@@ -35,7 +34,7 @@ def test_empty():
         "\u0034\u0035\u0036",
     ],
 )
-def test_string(fixture):
+def test_string(fixture: str) -> None:
     value = StringElement(fixture)
     check_write_read(value)
     assert value == fixture
@@ -53,7 +52,7 @@ def test_string(fixture):
         (NumericElement, 1.0),
     ],
 )
-def test_numbers(kls, fixture):
+def test_numbers(kls: Type[Any], fixture: int) -> None:
     value = kls(fixture)
     check_write_read(value)
     assert value == fixture
@@ -70,11 +69,11 @@ def test_numbers(kls, fixture):
         (BooleanElement, b"\x00\x01"),
     ],
 )
-def test_malformed_numbers(kls, fixture):
+def test_malformed_numbers(kls: Type[Any], fixture: bytes) -> None:
     kls.frombytes(fixture)
 
 
-def test_boolean():
+def test_boolean() -> None:
     value = BooleanElement(True)
     assert value.value is True
     assert value
@@ -83,8 +82,8 @@ def test_boolean():
     assert not value
 
 
-def test_list():
-    value = ListElement(range(10))
+def test_list() -> None:
+    value = ListElement(list(range(10)))  # type: ignore[arg-type]
     value.append(10)
     value.extend([11])
     value.insert(0, -1)
@@ -103,7 +102,7 @@ class Dummy(bytes, Enum):
     C = b"x"
 
 
-def test_dict():
+def test_dict() -> None:
     value = DictElement()
     value[Dummy.A] = "foo"
     assert value.get(Dummy.A) == value.get(b"x")
